import fire
import os
import subprocess
from pywinschedule.watcher import Watcher
from pywinschedule.config import HOME
from pywinschedule.utils.dbutils import DBList
from pywinschedule.utils.ipc import IPC
from pywinschedule.utils.process import kill_process

ipc=IPC()

watch_dirs_dblist=DBList(os.path.join(HOME,'watch_dirs.db'))

def generate_hash(s, times=1):
    assert times >= 1
    import hashlib
    m = hashlib.md5()

    def gen():
        m.update(s.encode('utf-8'))
        return m.hexdigest()[:10]

    for i in range(times):
        data = gen()
    return data


def startupdirectory():
    return os.path.join(os.path.expanduser('~'), r'AppData\Roaming\Microsoft\Windows\Start Menu\Programs\Startup')

STARTUP_PATH=startupdirectory()
def add_startup_command(command,script_name):
    bat_path = os.path.join(
        HOME,
        script_name + '.bat'
    )
    bat_content = command
    vbs_path = os.path.join(
        STARTUP_PATH,
        script_name + '.vbs'
    )
    vbs_content = 'CreateObject("Wscript.Shell").Run "%s", 0, True' % (bat_path)
    print('Writing to %s , %s' % (bat_path, vbs_path))
    with open(bat_path, 'w') as f:
        f.write(bat_content)
    with open(vbs_path, 'w') as f:
        f.write(vbs_content)

def remove_startup_command(script_name):
    bat_path = os.path.join(
        HOME,
        script_name + '.bat'
    )
    vbs_path = os.path.join(
        STARTUP_PATH,
        script_name + '.vbs'
    )
    os.remove(bat_path)
    os.remove(vbs_path)

def get_watch_dirs():
    return watch_dirs_dblist.list()
def add_watch_dir(watch_dir):
    watch_dir=os.path.abspath(watch_dir)
    watch_dirs_dblist.add(watch_dir)
    watch_dirs_dblist.commit()
def remove_watch_dir(watch_dir):
    watch_dir = os.path.abspath(watch_dir)
    if watch_dirs_dblist.contains(watch_dir):
        watch_dirs_dblist.remove(watch_dir)
        watch_dirs_dblist.commit()
    else:
        raise Exception('Path is unwatched : %s'%(watch_dir))

processes=[]

class CLI:
    @classmethod
    def enable(cls):
        add_startup_command(
            command='pywinschedule start',
            script_name='pywinschedule_start'
        )
    @classmethod
    def start(cls):
        for watch_dir in get_watch_dirs():
            args=['pywinschedule','watch',watch_dir]
            print(['pywinschedule','watch',watch_dir])

            proc=subprocess.Popen(['pywinschedule','watch',watch_dir])
            # proc=os.popen(' '.join(args))
            processes.append(proc)
        def kill_all_process(*args):
            print('Going to stop')
            for proc in processes:
                kill_process(proc.pid)
            quit(0)
        ipc.subscribe('stop_all_watcher', kill_all_process)
        ipc.listen()
    @classmethod
    def stop(cls):
        ipc.dispatch('stop_all_watcher','')
    @classmethod
    def disable(cls):
        remove_startup_command('pywinschedule_start')
    @classmethod
    def add(cls,path):
        if not os.path.exists(path):
            os.makedirs(path)
        assert os.path.isdir(path)
        add_watch_dir(path)
    @classmethod
    def list(cls):
        wds=get_watch_dirs()
        for wd in wds:
            print(wd)
    @classmethod
    def remove(cls,path):
        remove_watch_dir(path)
    @classmethod
    def watch(cls,path):
        print('Watch:',path)
        try:
            watcher = Watcher(path)
            watcher.start()
        except Exception as e:
            print(e)
            raise e
def main():
    fire.Fire(CLI)

if __name__ == '__main__':
    main()


import socket

class IPC:
    def __init__(self):
        self.HOST = '127.0.0.1'
        self.PORT = 22222
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.channels={}
    def subscribe(self,channel,handler):
        self.channels[channel]=handler
    def dispatch(self,channel,payload):
        sock=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((self.HOST,self.PORT))
        data=channel+":"+payload
        sock.sendall(data.encode())

    def listen(self):
        self.sock.bind((self.HOST, self.PORT))
        self.sock.listen(10)
        while True:
            coon, addr = self.sock.accept()
            print('Connected:', addr)
            while True:
                data = coon.recv(1024).decode()
                if not data:
                    break
                channel,payload=data.split(':',maxsplit=1)
                if channel in self.channels.keys():
                    self.channels[channel](payload)







import time
import os
import sys
import subprocess
import signal
import json
import threading
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler,FileModifiedEvent
from pywinschedule.config import HOME
from pathlib import Path
def json_load(f,encoding='utf-8',*args,**kwargs):
    import json
    with open(f,'r',encoding=encoding) as fp:
        return json.load(fp,*args,**kwargs)
def json_dump(obj,fp,encoding='utf-8',ensure_ascii=False,*args,**kwargs):
    import json
    with open(fp,'w',encoding=encoding) as f:
        json.dump(obj,f,ensure_ascii=ensure_ascii,*args,**kwargs)

def load_yaml(path):
    import yaml
    with open(path,encoding='utf-8') as f:
        return yaml.safe_load(f)
def dump_yaml(obj,path):
    import yaml
    with open(path,'w',encoding='utf-8') as f:
        yaml.dump(obj,f)

# 
# class IService:
#     def __init__(self):
#         self.config_path=Path(HOME)/'watcher-config.json'
#         self.lock_file=Path(HOME)/'watcher.lock'
#         self.socket_file=Path(HOME)/'watcher.socket'
#         if os.path.exists(self.config_path):
#             self.cfg=json_load(self.config_path)
#         else:
#             self.cfg={
#                 'watch_dirs':[]
#             }
#             json_dump(self.cfg,self.config_path)
#         self.observers={}
#     def start(self):
#         print('Start watching...')
#         for watch_dir in self.cfg['watch_dirs']:
#             watcher = Watcher(watch_dir)
#             o=watcher.start()
#             self.observers[watch_dir]=o
#     def stop(self):
#         print('Stop watching...')
#     def enable(self):
#         pass
#     def disable(self):
#         pass
# 
# class Service:
#     def __init__(self,name,root):
#         root=Path(root)
#         self.root=root
#         self.config_path=root/'%s-config.yaml'%(name)
#         self.lock_file=root/'%s.lock'%(name)
#         self.socket_file=root/'%s.socket'%(name)
#         if os.path.exists(self.config_path):
#             self.cfg=load_yaml(self.config_path)
#         else:
#             self.cfg=self.get_default_config()
#             dump_yaml(self.cfg,self.config_path)
#     def get_default_config(self):
#         return {}
#     def main(self):
#         while True:
#             time.sleep(1)
#             
#     def start(self):
#         print('Start watching...')
#         self.main()
#     def stop(self):
#         print('Stop watching...')
# 
#     def enable(self):
#         pass
#     def disable(self):
#         pass
# 
# 
# 







class Watcher(FileSystemEventHandler):
    def __init__(self,root_path:str):
        super(Watcher, self).__init__()
        self.root=root_path
        self.main_file=os.path.join(root_path,'main.py')
        self.process=None
    def run(self,file):
        if self.process:
            subprocess.Popen("TASKKILL /F /PID {pid} /T".format(pid=self.process.pid))
        os.chdir(self.root)
        self.process=subprocess.Popen([sys.executable,file])
    def start(self):
        self.run(self.main_file)
        self.watch()
    def on_modified(self, event:FileModifiedEvent):
        print('Modified:',event.src_path)
        if event.src_path.endswith('.py') and os.path.samefile(event.src_path,self.main_file):
            self.run(event.src_path)
    def watch(self):
        observer = Observer()
        observer.schedule(self, self.root, recursive=True)
        observer.start()
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
        observer.join()

def demo():
    watcher = Watcher(r'D:\Projects\PythonProjects\pywinschedule\demo\run')
    watcher.start()

if __name__ == "__main__":
    demo()